@@ -1,5 +1,3 @@
-import matplotlib.pyplot as plt
-from mpl_toolkits.mplot3d import Axes3D
 import re
 from struct import unpack,pack
 import time
@@ -11,16 +9,12 @@ import snap7
 import snap7.types
 from snap7.common import check_error, load_library, ipv4
 
-import threading
 import numpy as np
 from functools import partial
 
 from speedbotlib.common.cache import dson
-
-# import logging
-# logging.basicConfig(level = logging.INFO)
-# logger = logging.getLogger(__name__)
-
+from copy import deepcopy
+                
 def rotate(origin_pos,pos,degree):
     from math import pi,cos,sin
     a = pi / 180 * degree
@@ -30,7 +24,7 @@ def rotate(origin_pos,pos,degree):
     return x2,y2,z2
 
 class Truss():
-    def __init__(self,name,db_dir,port,area_x,area_y,area_z):
+    def __init__(self,view,name,db_dir,db_indices,port,area_x,area_y,area_z,interference_area_x,reset_x):
         self.area_x,self.area_y,self.area_z = area_x,area_y,area_z
         self.plate_dir = f'{db_dir}/plates_sorting'
         self.status_file = f'{db_dir}/status/{name}.json'
@@ -40,42 +34,43 @@ class Truss():
         self.dbs = dict()
         self.db0 = (snap7.types.wordlen_to_ctypes[snap7.types.S7WLByte] * 200)()
         self.db1 = (snap7.types.wordlen_to_ctypes[snap7.types.S7WLByte] * 200)()
-        self.server.register_area(snap7.types.srvAreaDB, 0, self.db0)
-        self.server.register_area(snap7.types.srvAreaDB, 1, self.db1)
+        self.db2 = []
+        self.db3 = []
+        self.server.register_area(snap7.types.srvAreaDB, db_indices[0], self.db0)
+        self.server.register_area(snap7.types.srvAreaDB, db_indices[1], self.db1)
         self.server.start(tcpport=port)
 
         self.name = name
-        self.graph = plt.figure(name)
-        self.view = self.graph.add_subplot(111,projection='3d')
-
-        plt.show(block=False)
-        plt.ion()
-
+        self.view = view
+        self.db_indices = db_indices
+        self.interference_area_x = interference_area_x
+        self.reset_x = reset_x
+        
         self.loop = asyncio.get_event_loop()
         pass
     def __del__(self):
-        plt.ioff()
         pass
     def power_on(self):
-        self.db0[0] = 88 # A
-        self.db0[1] = 88 # B
-        self.db1[34] = 0b00000010 # fault,writeable,paused,safe_a,safe_b,finished_a,finished_b,synced
-        
-        self.x = [-3400,3200]
+        self.db0[0] = 88 
+        self.db0[1] = 88 
+        self.db1[34] = 0b00000010 
+        self.db1[35] = 0b00000000 
+
+        self.x = deepcopy(self.reset_x)
         self.y = [self.area_y[0],self.area_y[0]]
         self.z = [self.area_z[1],self.area_z[1]]
         self.r = [0,0]
         
         self.db1[2],self.db1[3],self.db1[4],self.db1[5] = pack('!f',self.x[0])
-        self.db1[6],self.db1[7],self.db1[8],self.db1[9] = pack('!f',self.y[0])        
+        self.db1[6],self.db1[7],self.db1[8],self.db1[9] = pack('!f',self.y[0])
         self.db1[10],self.db1[11],self.db1[12],self.db1[13] = pack('!f',self.z[0])
         self.db1[18],self.db1[19],self.db1[20],self.db1[21] = pack('!f',self.x[1])
         self.db1[22],self.db1[23],self.db1[24],self.db1[25] = pack('!f',self.y[1])
         self.db1[26],self.db1[27],self.db1[28],self.db1[29] = pack('!f',self.z[1])
         pass
     def emergency_stop(self):
-        self.db0[0] = 0 # A
-        self.db0[1] = 0 # B
+        self.db0[0] = 0 
+        self.db0[1] = 0 
         pass
     def pause(self):
         pass
@@ -85,17 +80,23 @@ class Truss():
         pass
     async def moving_x(self,index):
         if index == 0:
-            end = unpack('!f',pack('4b',self.db0[38],self.db0[39],self.db0[40],self.db0[41]))[0]
-            start = unpack('!f',pack('4b',self.db1[2],self.db1[3],self.db1[4],self.db1[5]))[0]
+            x_end = unpack('!f',pack('4b',self.db0[38],self.db0[39],self.db0[40],self.db0[41]))[0]
+            x_start = unpack('!f',pack('4b',self.db1[2],self.db1[3],self.db1[4],self.db1[5]))[0]
         elif index == 1:
-            end = unpack('!f',pack('4b',self.db0[70],self.db0[71],self.db0[72],self.db0[73]))[0]
-            start = unpack('!f',pack('4b',self.db1[18],self.db1[19],self.db1[20],self.db1[21]))[0]
+            x_end = unpack('!f',pack('4b',self.db0[70],self.db0[71],self.db0[72],self.db0[73]))[0]
+            x_start = unpack('!f',pack('4b',self.db1[18],self.db1[19],self.db1[20],self.db1[21]))[0]
 
-        for interval in np.linspace(start,end,int(abs(start - end) / 100)):
+        for interval in np.linspace(x_start,x_end,int(abs(x_start - x_end) / 200)):
             if index == 0: self.db1[2],self.db1[3],self.db1[4],self.db1[5] = pack('!f',interval)
             elif index == 1: self.db1[18],self.db1[19],self.db1[20],self.db1[21] = pack('!f',interval)
             self.x[index] = interval
             await asyncio.sleep(0.01)
+
+        if self.db_indices[0] == 27 and index == 0 or self.db_indices[0] == 13 and index == 1:
+            if self.interference_area_x[0] <= x_end and x_end <= self.interference_area_x[1]: 
+                self.db1[35] = self.db1[35] | 0b00000010
+            else: 
+                self.db1[35] = self.db1[35] & 0b11111101
         pass
     async def moving_y(self,index):
         if index == 0:
@@ -104,7 +105,7 @@ class Truss():
         elif index == 1:
             end = unpack('!f',pack('4b',self.db0[74],self.db0[75],self.db0[76],self.db0[77]))[0]
             start = unpack('!f',pack('4b',self.db1[22],self.db1[23],self.db1[24],self.db1[25]))[0]
-        for interval in np.linspace(start,end,int(abs(start - end) / 100)):
+        for interval in np.linspace(start,end,int(abs(start - end) / 200)):
             if index == 0: self.db1[6],self.db1[7],self.db1[8],self.db1[9] = pack('!f',interval)
             elif index == 1: self.db1[22],self.db1[23],self.db1[24],self.db1[25] = pack('!f',interval)
             self.y[index] = interval
@@ -117,11 +118,11 @@ class Truss():
         elif index == 1:
             end = unpack('!f',pack('4b',self.db0[78],self.db0[79],self.db0[80],self.db0[81]))[0]
             start = unpack('!f',pack('4b',self.db1[26],self.db1[27],self.db1[28],self.db1[29]))[0]
-        for interval in np.linspace(start,end,int(abs(start - end) / 10)):
+        for interval in np.linspace(start,end,int(abs(start - end) / 100)):
             if index == 0: self.db1[10],self.db1[11],self.db1[12],self.db1[13] = pack('!f',interval)
             elif index == 1: self.db1[26],self.db1[27],self.db1[28],self.db1[29] = pack('!f',interval)
             self.z[index] = interval
-            await asyncio.sleep(0.001)
+            await asyncio.sleep(0.01)
     async def moving_r(self,index):
         if index == 0:
             end = unpack('!f',pack('4b',self.db0[50],self.db0[51],self.db0[52],self.db0[53]))[0]
@@ -129,11 +130,11 @@ class Truss():
         elif index == 1:
             end = unpack('!f',pack('4b',self.db0[82],self.db0[83],self.db0[84],self.db0[85]))[0]
             start = unpack('!f',pack('4b',self.db1[30],self.db1[31],self.db1[32],self.db1[33]))[0]
-        for interval in np.linspace(start,end,int(abs(start - end) / 10)):
+        for interval in np.linspace(start,end,int(abs(start - end) / 50)):
             if index == 0: self.db1[14],self.db1[15],self.db1[16],self.db1[17] = pack('!f',interval)
             elif index == 1: self.db1[30],self.db1[31],self.db1[32],self.db1[33] = pack('!f',interval)
             self.r[index] = interval
-            await asyncio.sleep(0.001)
+            await asyncio.sleep(0.01)
     async def picking(self,index,synced,index2):
         if index == 0:
             x_start = unpack('!f',pack('4b',self.db1[2],self.db1[3],self.db1[4],self.db1[5]))[0]
@@ -163,65 +164,72 @@ class Truss():
             r2_end = unpack('!f',pack('4b',self.db0[98],self.db0[99],self.db0[100],self.db0[101]))[0]
 
         async def x(self,start,end):
-            for interval in np.linspace(start,end,int(abs(start - end) / 100)+1):
+            for interval in np.linspace(start,end,int(abs(start - end) / 200)+1):
                 if index == 0: self.db1[2],self.db1[3],self.db1[4],self.db1[5] = pack('!f',interval)
                 elif index == 1: self.db1[18],self.db1[19],self.db1[20],self.db1[21] = pack('!f',interval)
                 self.x[index] = interval
                 await asyncio.sleep(0.01)
         async def y(self,start,end):
-            for interval in np.linspace(start,end,int(abs(start - end) / 100)+1):
+            for interval in np.linspace(start,end,int(abs(start - end) / 200)+1):
                 if index == 0: self.db1[6],self.db1[7],self.db1[8],self.db1[9] = pack('!f',interval)
                 elif index == 1: self.db1[22],self.db1[23],self.db1[24],self.db1[25] = pack('!f',interval)
                 self.y[index] = interval
                 await asyncio.sleep(0.01)
         async def z(self,start,end):
-            for interval in np.linspace(start,end,int(abs(start - end) / 40)+1):
+            for interval in np.linspace(start,end,int(abs(start - end) / 100)+1):
                 if index == 0: self.db1[10],self.db1[11],self.db1[12],self.db1[13] = pack('!f',interval)
                 elif index == 1: self.db1[26],self.db1[27],self.db1[28],self.db1[29] = pack('!f',interval)
                 self.z[index] = interval
-                await asyncio.sleep(0.001)
+                await asyncio.sleep(0.01)
         async def r(self,start,end):
-            for interval in np.linspace(start,end,int(abs(start - end) / 10)+1):
+            for interval in np.linspace(start,end,int(abs(start - end) / 50)+1):
                 if index == 0: self.db1[14],self.db1[15],self.db1[16],self.db1[17] = pack('!f',interval)
                 elif index == 1: self.db1[30],self.db1[31],self.db1[32],self.db1[33] = pack('!f',interval)
                 self.r[index] = interval
-                await asyncio.sleep(0.001)
+                await asyncio.sleep(0.01)
+
+        if self.db_indices[0] == 27 and index == 0 or self.db_indices[0] == 13 and index == 1:
+            if self.interference_area_x[0] <= x_end and x_end <= self.interference_area_x[1]: 
+                self.db1[35] = self.db1[35] | 0b00000010
+            else: 
+                self.db1[35] = self.db1[35] & 0b11111101
 
         await self.loop.create_task(asyncio.wait([x(self,x_start,x_end),y(self,y_start,y_end),r(self,r_start,r_end)]))
         if synced: 
             synced[index] = 1
-            while synced[index] > synced[index2]: await asyncio.sleep(0.1)
+            while synced[index] > synced[index2]: await asyncio.sleep(0.01)
         await self.loop.create_task(z(self,z_start,z_end))
         if synced: 
             synced[index] = 2
-            while synced[index] > synced[index2]: await asyncio.sleep(0.1)
+            while synced[index] > synced[index2]: await asyncio.sleep(0.01)
         await self.loop.create_task(z(self,z_end,self.area_z[1]))
         if synced: 
             synced[index]= 3
-            while synced[index] > synced[index2]: await asyncio.sleep(0.1)
+            while synced[index] > synced[index2]: await asyncio.sleep(0.01)
+
+        if self.db_indices[0] == 27 and index == 0 or self.db_indices[0] == 13 and index == 1:
+            if self.interference_area_x[0] <= x2_end and x2_end <= self.interference_area_x[1]: 
+                self.db1[35] = self.db1[35] | 0b00000010
+            else: 
+                self.db1[35] = self.db1[35] & 0b11111101
+
         await self.loop.create_task(asyncio.wait([x(self,x_end,x2_end),y(self,y_end,y2_end),r(self,r_end,r2_end)]))
         if synced: 
             synced[index] = 4
-            while synced[index] > synced[index2]: await asyncio.sleep(0.1)
+            while synced[index] > synced[index2]: await asyncio.sleep(0.01)
         await self.loop.create_task(z(self,self.area_z[1],z2_end))
         if synced: 
             synced[index] = 5
-            while synced[index] > synced[index2]: await asyncio.sleep(0.1)
+            while synced[index] > synced[index2]: await asyncio.sleep(0.01)
         await self.loop.create_task(z(self,z2_end,self.area_z[1]))
         if synced: 
             synced[index] = 6
-            while synced[index] > synced[index2]: await asyncio.sleep(0.1)
+            while synced[index] > synced[index2]: await asyncio.sleep(0.01)
         pass
     async def update(self):
-        if not plt.fignum_exists(self.name):
-            print('终止！')
-            self.loop.stop()
-            return
-            
         while True:
             event = self.server.pick_event()
             if not event: break
-
             request = self.server.event_text(event)
             
             m = re.match(r'[0-9]{4}-[0-9]{2}-[0-9]{2}\s[0-9]{2}:[0-9]{2}:[0-9]{2} \[[\d]+\.[\d]+\.[\d]+.[\d]+\] (\w+) request, Area : DB(\d+), Start : (\d+), Size : (\d+) --> OK',request)
@@ -230,62 +238,75 @@ class Truss():
             area = int(m.group(2))
             index = int(m.group(3))
             size = int(m.group(4))
-            if command != 'Write': continue
-            if index > 1: continue
-            print(request)
             action = self.db0[index]
-            def finish(db,index,f):
-                db[index] = 88
+            
+            if command != 'Write': continue
+            
+            if index == 2 and self.db2:
+                if action & 0b00010000: #上锁
+                    if self.db2[2] & 0b00010000 and self.db3[35] & 0b00000001:
+                        self.db1[35] = self.db1[35] & 0b11111110
+                    else:
+                        self.db1[35] = self.db1[35] | 0b00000001
+                else: #解锁
+                    self.db1[35] = self.db1[35] & 0b11111110
+                    if self.db2[2] & 0b00010000:
+                        self.db3[35] = self.db3[35] | 0b00000001
+                pass
+            elif index <= 1: 
+                def finish(db,index,f):
+                    db[index] = 88
 
-            if action == 12:
-                print('升降指令')
-                self.loop.create_task(self.moving_z(index)).add_done_callback(partial(finish,self.db0,index))
-                pass
-            elif action == 11:
-                print('平移指令')
-                self.loop.create_task(
-                    asyncio.wait([
-                            self.loop.create_task(self.moving_x(index)),
-                            self.loop.create_task(self.moving_y(index)),
-                            self.loop.create_task(self.moving_r(index))
-                        ])).add_done_callback(partial(finish,self.db0,index))
-                pass
-            elif action == 1:
-                print('单臂抓取指令')
-                self.loop.create_task(self.picking(index,None,None)).add_done_callback(partial(finish,self.db0,index))
-                pass
-            elif action == 2:
-                print('双臂抓取指令')
-                synced = [0,0]
-                self.loop.create_task(self.picking(0,synced,1)).add_done_callback(partial(finish,self.db0,0))
-                self.loop.create_task(self.picking(1,synced,0)).add_done_callback(partial(finish,self.db0,1))
-                pass
-            elif action == 3:
-                print('复位指令')
-                if index == 0:
-                    self.db0[38],self.db0[39],self.db0[40],self.db0[41] = pack('!f',-3400)
-                    self.db0[42],self.db0[43],self.db0[44],self.db0[45] = pack('!f',self.area_y[0])
-                    self.db0[46],self.db0[47],self.db0[48],self.db0[49] = pack('!f',self.area_z[1])
-                    self.db0[50],self.db0[51],self.db0[52],self.db0[53] = pack('!f',0)
-                elif index == 1:
-                    self.db0[70],self.db0[71],self.db0[72],self.db0[73] = pack('!f',3200)
-                    self.db0[74],self.db0[75],self.db0[76],self.db0[77] = pack('!f',self.area_y[0])
-                    self.db0[78],self.db0[79],self.db0[80],self.db0[81] = pack('!f',self.area_z[1])
-                    self.db0[82],self.db0[83],self.db0[85],self.db0[85] = pack('!f',0)
+                if action == 1:
+                    # print('单臂抓取指令')
+                    self.loop.create_task(self.picking(index,None,None)).add_done_callback(partial(finish,self.db0,index))
+                    pass
+                elif action == 2:
+                    # print('双臂抓取指令')
+                    synced = [0,0]
+                    self.loop.create_task(self.picking(0,synced,1)).add_done_callback(partial(finish,self.db0,0))
+                    self.loop.create_task(self.picking(1,synced,0)).add_done_callback(partial(finish,self.db0,1))
+                    pass
+                elif action == 3:
+                    # print('复位指令')
+                    if index == 0:
+                        self.db0[38],self.db0[39],self.db0[40],self.db0[41] = pack('!f',self.reset_x[0])
+                        self.db0[42],self.db0[43],self.db0[44],self.db0[45] = pack('!f',self.area_y[0])
+                        self.db0[46],self.db0[47],self.db0[48],self.db0[49] = pack('!f',self.area_z[1])
+                        self.db0[50],self.db0[51],self.db0[52],self.db0[53] = pack('!f',0)
+                    elif index == 1:
+                        self.db0[70],self.db0[71],self.db0[72],self.db0[73] = pack('!f',self.reset_x[1])
+                        self.db0[74],self.db0[75],self.db0[76],self.db0[77] = pack('!f',self.area_y[0])
+                        self.db0[78],self.db0[79],self.db0[80],self.db0[81] = pack('!f',self.area_z[1])
+                        self.db0[82],self.db0[83],self.db0[85],self.db0[85] = pack('!f',0)
 
-                self.loop.create_task(
-                    asyncio.wait([
-                            self.loop.create_task(self.moving_x(index)),
-                            self.loop.create_task(self.moving_y(index)),
-                            self.loop.create_task(self.moving_z(index)),
-                            self.loop.create_task(self.moving_r(index))
-                        ])).add_done_callback(partial(finish,self.db0,index))
-                pass
+                    self.loop.create_task(
+                        asyncio.wait([
+                                self.loop.create_task(self.moving_x(index)),
+                                self.loop.create_task(self.moving_y(index)),
+                                self.loop.create_task(self.moving_z(index)),
+                                self.loop.create_task(self.moving_r(index))
+                            ])).add_done_callback(partial(finish,self.db0,index))
+                    pass
+                elif action == 11:
+                    # print('平移指令')
+                    self.loop.create_task(
+                        asyncio.wait([
+                                self.loop.create_task(self.moving_x(index)),
+                                self.loop.create_task(self.moving_y(index)),
+                                self.loop.create_task(self.moving_r(index))
+                            ])).add_done_callback(partial(finish,self.db0,index))
+                    pass
+                elif action == 12:
+                    # print('升降指令')
+                    self.loop.create_task(self.moving_z(index)).add_done_callback(partial(finish,self.db0,index))
+                    pass
+                else:
+                    pass
 
             self.status = dson(self.status_file)
             self.params = dson(self.params_file)
-            if 'plate_name' not in self.status:
-                continue
+            if 'plate_name' not in self.status: continue
             plate_name = self.status['plate_name']
             self.plate = dson(f'{self.plate_dir}/{plate_name}.json')
 
@@ -301,7 +322,7 @@ class Truss():
 
         try:
             #画钢板
-            plate_corner,plate_degree = self.status['corner'],self.status['degree']
+            plate_corner,plate_degree = self.status['corner']
             
             plate_size = self.plate['size']
             self.plate_right,self.plate_top = plate_corner
@@ -330,10 +351,9 @@ class Truss():
             self.view.text(p3[0],p3[1],p3[2],s=str(drop_pos))
 
             if self.plate['finished']:
-                drop_positions = self.plate['tiling'][-1]
+                drop_positions = self.plate['tiling']
             else:
-                tiling_index = self.plate['tiling_index']
-                drop_positions = self.plate['tiling'][tiling_index]
+                drop_positions = self.plate['tiling']
 
             for part in self.plate['parts']:
                 from copy import deepcopy
@@ -457,33 +477,22 @@ class Truss():
                     self.rectangle(p0, p1, p2, p3)
         except:pass
                     
-        #桁架
         self.view.plot([self.x[0],self.x[0]],[self.area_y[0],self.area_y[1]],[self.area_z[1],self.area_z[1]],c='red',linewidth=1)
         self.view.plot([self.x[1],self.x[1]],[self.area_y[0],self.area_y[1]],[self.area_z[1],self.area_z[1]],c='red',linewidth=1)
         self.view.plot([self.x[0],self.x[0]],[self.y[0],self.y[0]],[self.z[0],self.area_z[1]],c='red',linewidth=1)
         self.view.plot([self.x[1],self.x[1]],[self.y[1],self.y[1]],[self.z[1],self.area_z[1]],c='red',linewidth=1)
         
-        #抓手A
         x0,y0,_=rotate([self.x[0],self.y[0],0],[self.x[0]-750,self.y[0],0],-self.r[0])
         x1,y1,_=rotate([self.x[0],self.y[0],0],[self.x[0]+750,self.y[0],0],-self.r[0])
         self.view.plot([x0,x1],[y0,y1],[self.z[0],self.z[0]],c='blue',linewidth=1)
 
-        #抓手B
         x0,y0,_=rotate([self.x[1],self.y[1],0],[self.x[1]-750,self.y[1],0],-self.r[1])
         x1,y1,_=rotate([self.x[1],self.y[1],0],[self.x[1]+750,self.y[1],0],-self.r[1])
         self.view.plot([x0,x1],[y0,y1],[self.z[1],self.z[1]],c='blue',linewidth=1)
 
-        #位置
         self.view.text(self.x[0],self.y[0],self.z[0],str([int(self.x[0]),int(self.y[0]),int(self.z[0]),int(self.r[0])]))
         self.view.text(self.x[1],self.y[1],self.z[1],str([int(self.x[1]),int(self.y[1]),int(self.z[1]),int(self.r[1])]))
-
-        self.graph.canvas.draw()
-        self.graph.canvas.flush_events()
-        await asyncio.sleep(0.001)
-        
-        self.loop.create_task(self.update())
     def rectangle(self,p0,p1,p2,p3,c=None,s=''):
-        #可视化：钢板
         self.view.plot(
             [p0[0],p1[0],p2[0],p3[0],p0[0]],
             [p0[1],p1[1],p2[1],p3[1],p0[1]],
